/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui.util;

import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ComponentAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.JDialog;
import javax.swing.JFrame;

import com.backup.zipsnap.ng.gui.GUI;

public class WindowsManager extends JDialog {

	private static final long serialVersionUID = 7507476642964639287L;
	private GUI gui;
	private ArrayList<JFrame> openedWindows = new ArrayList<JFrame>();
	
	
	/*
	 * init the window manager
	 */
	public WindowsManager(GUI gui) {
		this.gui = gui;
	}
	
	/*
	 * show a JFrame through the window manager
	 */
	public JFrame show(final JFrame window) {
		/* determine default x and y coordinates*/
		int x = 0;
		int y = 0;
		if (window.getClass().toString().contains("DialogFrame")) {
			x = gui.mainWindow.getWidth()/2;
			y = gui.mainWindow.getHeight()/2;
		} else {
			x = gui.mainWindow.getX()+50;
			y = gui.mainWindow.getY()+50;
		}
		/* determine x and y not busy by another window */
		while (isPositionBusy(x,y)) {
			x = x+50;
			y = y+50;
		}
		
		/* set the window's position and show it */
		window.setBounds(x, y, window.getWidth(), window.getHeight());
		window.setVisible(true);

		/* add the window to the opened windows list*/
		openedWindows.add(window);
		/* add close callback */ 
		window.addWindowListener(new WindowAdapter() {
		    public void windowClosed(WindowEvent e) {
		    	closeWindow(window);
		    }
		});
		window.addComponentListener(new ComponentAdapter() {
			public void componentHidden(ComponentEvent e) {
				closeWindow(window);
			}
		});
		return window;
	}
	
	/*
	 * check if in a given position there is already an opened window
	 */
	private boolean isPositionBusy(int x, int y) {
		for (JFrame window : openedWindows) {
			if (window.getX() == x && window.getY() == y) return true;
		}
		return false;
	}
	
	/*
	 * remove a window from the opened windows
	 */
	private void closeWindow(JFrame window) {
    	if (openedWindows.contains(window)) {
    		openedWindows.remove(window);
    	}
	}
	
	/*
	 * close all the opened windows
	 */
	@SuppressWarnings("unchecked")
	public void closeAll() {
		for (JFrame window : (ArrayList<JFrame>) openedWindows.clone()) window.dispose();
	}
	

}
