/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.job.util;

import java.util.HashMap;

/*
 * Keeps track of multiple elapsed time
 */
public class Timer
{
  /* store for each timer the starting time*/
  private HashMap<String,Long> timers = new HashMap<String,Long>();
  private HashMap<String,Long> pauseInMilliseconds = new HashMap<String,Long>();
  
  /*
   * initiate the timers
   */
  public Timer()
  {
  }
  
    
  /*
   * Start a timer with the given name
   */
  public void start(String item) {
    if (timers.containsKey(item)) timers.remove(item);
    timers.put(item,System.nanoTime());
  }

  /*
   * Pause a timer with the given name
   */
  public void pause(String item) {
    if (! timers.containsKey(item)) return;
    /* start counting how lon the pause will be*/
    start("__PAUSE__"+item);
  }
  
  
  /*
   * Resume a timer with the given name
   */
  public void resume(String item) {
    if (! timers.containsKey(item)) return;
    /* add the number of seconds of this pause to the others if any */
    long previousPause = 0;
    if (pauseInMilliseconds.containsKey(item)) previousPause = pauseInMilliseconds.get(item);
    pauseInMilliseconds.put(item,previousPause+stopInMilliseconds("__PAUSE__"+item));
  }
      
  /*
   * Stop a timer and return the elapsed time as a string
   */
  public String stop(String item) {
    long seconds = stopInSeconds(item);
    if (seconds == -1) return "N.A.";
    return seconds2string(seconds);
  }
  
  /*
   * Stop a timer and return the elapsed time in milliseconds
   */
  public long stopInMilliseconds(String item) {
    if (!timers.containsKey(item)) return -1;
    double milliseconds = ((double)(System.nanoTime() - (long)timers.get(item)))/ 1000000;
    /* sub the seconds in a pause status if any */
    if (pauseInMilliseconds.containsKey(item)) milliseconds -= pauseInMilliseconds.get(item);
    return (long) milliseconds;
  }
  
  /*
   * Stop a timer and return the elapsed time in seconds
   */
  public long stopInSeconds(String item) {
	return stopInMilliseconds(item)/1000;
  }
  
  /*
   * convert number of seconds into an elapsed string
   */
  public static String seconds2string(long seconds) {
    int hours = (int) seconds / 3600;
    int remainder = (int) seconds - hours * 3600;
    int mins = remainder / 60;
    remainder = remainder - mins * 60;
    int secs = remainder;
    String elapsed  = (hours > 0 ? hours+"h " : "") + (mins > 0 ? mins+"m " : "") + secs+"s";
    return elapsed;
  }
}